/**
 * The different values of "environment" in the SDK
 * This is used for, among other things, determining the root URL for backend calls
 */
export declare enum Environment {
    LOCAL = "local",
    E2E = "e2e",
    DEV = "dev",
    QA = "qa",
    PROD = "prod"
}
/**
 * Commonly used status codes (for {@link AsyncStatus})
 * Currently we only create AsyncStatus objects for success states; errors are thrown
 * and bubble up instead, hence why only 20X status codes are represented here.
 */
export declare enum StatusCode {
    SUCCESS = 200,
    ACCEPTED = 202
}
/**
 * The different values of severity for logging
 */
export declare enum Severity {
    INFO = "info",
    WARNING = "warning",
    ERROR = "error"
}
/**
 * The different Entity units for experiments
 */
export declare enum EntityType {
    USER = "USER",
    GROUP = "GROUP",
    ORG = "ORG"
}
/**
 * An optional type interface consumers can pass in during initializiation
 * to enforce enums for Experiment data. It is highly encouraged to pass this typing
 * in.
 */
export declare type TypeData = {
    /**
     * Enum representing experiment names. Should match expected values for
     * {@link ExperimentAssignment.testName}
     * @example
     * enum ExperimentNames {
     *  EXP_ONE = 'expOne',
     * };
     * const types: TypeData = {
     *  experimentNames: ExperimentNames,
     *  ...
     * }
     */
    experimentNames: string;
    /**
     * Enum representing experiment variant names. Should match expected values for
     * {@link ExperimentAssignment.testGroupId}
     * @example
     * enum ExperimentVariantNames {
     *  EXP_VARIANT_ONE = 'expVariantOne',
     * };
     * const types: TypeData = {
     *  experimentVariantNames: ExperimentVariantNames,
     *  ...
     * }
     */
    experimentVariantNames: string;
    /**
     * Enum representing experiment attribute names
     * @example
     * enum ExperimentAttributeNames {
     *  SHOW_LEFT_NAV_LINK = 'showLeftNavLink',
     * };
     * const types: TypeData = {
     *  experimentAttributeNames: ExperimentAttributeNames,
     *  ...
     * }
     */
    experimentAttributeNames: string;
    /**
     * Enum representing allocation point names
     * @example
     * enum AllocationPointNames {
     *  CLUSTER_CARD_PAGE = 'CLUSTER_CARD_PAGE',
     * };
     * const types: TypeData = {
     *  allocationPointNames: AllocationPointNames,
     *  ...
     * }
     */
    allocationPointNames: string;
    /**
     * Type/Interface containing key/value pairs of properties that may be included in the "Experiment Viewed event
     * @example
     * export interface ExperimentViewedBaseEvent {
     *   test_name: TestName;
     *   test_group_name: TestGroupName;
     * }
     * interface ExperimentViewedPropertiesForExperimentA {
     *   aCustomProp: string;
     * }
     * interface ExperimentViewedPropertiesForExperimentB {
     *   aSecondCustomProp: string;
     *   aThirdCustomProp: string;
     * }
     * type ExperimentViewedAdditionalProperties =
     *   | ExperimentViewedPropertiesForExperimentA | ExperimentViewedPropertiesForExperimentB;
     * type ExperimentViewedEventProps =
     *   | ExperimentViewedBaseEvent
     *   | (ExperimentViewedBaseEvent & ExperimentViewedAdditionalProperties);
     *
     * const types: TypesToPassIn = {
     *   experimentViewedProps: ExperimentViewedEventProps,
     *   ...
     * }
     */
    experimentViewedProps: {
        [name: string]: string | number;
    };
    /**
     * Enum representing different teams that can be assigned application logs
     * @example
     * enum LoggerTeams {
     *  Team1 = 'Test Team 1',
     * };
     * const types: TypeData = {
     *  loggerTeams: LoggerTeams,
     *  ...
     * }
     */
    loggerTeams: string;
    /**
     * Enum representing different application log severity levels. The SDK expects
     * to have at least 3 values defined for this: ERROR, WARNING, and INFO.
     * @example
     * enum LoggerSeverities {
     *  ERROR = 'error',
     *  WARNING = 'warning',
     *  INFO = 'info',
     * };
     * const types: TypeData = {
     *  loggerSeverities: LoggerSeverities,
     *  ...
     * }
     */
    loggerSeverities: string;
};
/**
 * A simple object representing the status of a response from an async call
 */
export interface AsyncStatus {
    statusCode: StatusCode;
    message?: string;
}
/**
 * An experiment assignment for a given entity and experiment.
 * This represents the raw assignment data fed into {@link ExperimentSDK.addExistingExperimentData}.
 * (FYI The below interface is taken from MMS: js/admin/controlpanel/types/abTest.ts)
 */
export interface ServerAssignment<ExperimentNames, ExperimentVariantNames> {
    /**
     * The date of the experiment assignment, in ms.
     * @example
     * "1658340252128"
     */
    assignmentDate: string;
    /**
     * The ID of the entity assigned.
     * @example
     * "621509aae911cb2e37ba91ca"
     */
    entityId: string;
    /**
     * The "type" of entity. Experiments are configured to allocate specific entity types.
     * @example
     * "ORG"
     */
    entityType: EntityType;
    /**
     * ID of the experiment assignment.
     * @example
     * "62d8439c9ce6473793cff6ca"
     */
    id: string;
    /**
     * Experiment tag, set in the experiment configuration. Experiments on the same tag
     * are mutually exclusive.
     * @example
     * "activation"
     */
    tag: string;
    /**
     * The ID of the experiment configuration.
     * @example
     * "62bb810b61be7d017fcc2198"
     */
    testId: string;
    /**
     * The name of the experiment. Should be a value from the {@link TypeData.experimentNames} enum.
     * @example
     * "ATLAS_SEARCH_IN_PROJECT_SIDE_NAV_20220620"
     */
    testName: ExperimentNames;
    /**
     * The name of the assigned experiment variant for the entity.
     * (Confusingly is NOT an ObjectId.)
     * If defined, should be a value from the {@link TypeData.experimentVariantNames} enum.
     * The "/assign" server response will have this always
     * defined as a string, but the "/params" and "/assignByPoint" endpoints may have "null" for an
     * experiment, which would mean the entity is NOT allocated into the experiment
     * (aka not in any of the variants, including control).
     *
     * @example
     * "atlasSearchProjectSideNavControl"
     */
    testGroupId: ExperimentVariantNames | null;
    /**
     * The ID of the assigned experiment variant for the entity.
     * The "/assign" server response will have this always
     * defined as a string, but the "/params" and "/assignByPoint" endpoints may have "null" for an
     * experiment, which would mean the entity is NOT allocated into the experiment
     * (aka not in any of the variants, including control).
     *
     * @example
     * "62bb810b61be7d017fcc2196"
     */
    testGroupDatabaseId: string | null;
}
/**
 * An experiment assignment for a given entity and experiment in the SDK.
 * This represents an assignment object in {@link ExperimentSDK.#assignmentsMap}
 */
export interface SDKAssignment<ExperimentNames, ExperimentVariantNames> {
    /**
     * Critical data about the specific entity's assignment
     */
    assignmentData: {
        /**
         * Represents if the entity has been allocated into the experiment. Should ALWAYS be used
         * to check if an entity is allocated in an experiment--just checking for the presence of the experiment name key
         * in SDK state isn't enough.
         */
        isInSample: boolean;
        /**
         * If the entity is allocated, the variant name they are assigned to.
         * Can be null if they are not allocated in the experiment.
         */
        variant: ExperimentVariantNames | null;
    };
    /**
     * All the information from the server about the experiment and the entity's assignment
     */
    experimentData: ServerAssignment<ExperimentNames, ExperimentVariantNames>;
}
/**
 * An experiment attribute from the server. The values are all strings.
 * This represents the raw experiment attribute data fed into {@link ExperimentSDK.addExistingExperimentData}
 * (which takes cares of transforming the strings into booleans and numbers as needed).
 * (FYI The below interface is taken from MMS: js/admin/controlpanel/types/abTest.ts)
 */
export declare type ServerExperimentAttributes<ExperimentAttributeNames extends string> = {
    [key in ExperimentAttributeNames]?: string;
};
/**
 * Represents the types an Experiment Attribute in an experiment configuration can be.
 */
export declare type ExperimentAttributeValue = string | boolean | number;
/**
 * An experiment attribute in the SDK.
 * This represents an experiment attribute object in {@link ExperimentSDK.#experimentAttributesMap}
 */
export declare type SDKExperimentAttributes<ExperimentAttributeNames extends string> = {
    [key in ExperimentAttributeNames]?: ExperimentAttributeValue;
};
/**
 * A set of ID values (org, group, and user). Each ID is treated as a distinct
 * entity for experiments. Experiment configuration dictates which entity ID it
 * will use for allocation.
 */
export interface EntityIds {
    /**
     * A string representing an organization's ID
     * @example
     * "621509aae911cb2e37ba91ca"
     */
    orgId?: string;
    /**
     * A string representing a group/project's ID
     * @example
     * "621509aae911cb2e37ba91ca"
     */
    groupId?: string;
    /**
     * A string representing a user's ID
     * @example
     * "621509aae911cb2e37ba91ca"
     */
    userId?: string;
}
/**
 * The interface for a log message + metadata, for use with the {@link Logger}.
 */
export interface Log<LoggerTeams, LoggerSeverities> {
    logMessage: string;
    severity: LoggerSeverities;
    team?: LoggerTeams;
}
/**
 * Data ingested by the internal #log function in the SDK.
 */
export interface SDKLogOptions<LoggerTeams, LoggerSeverities> extends Log<LoggerTeams, LoggerSeverities> {
    additionalLogInfo?: Record<string, string | null | undefined>;
}
/**
 * A Logger object that can be used to monitor SDK health.
 */
export interface Logger<LoggerTeams, LoggerSeverities> {
    /**
     * @description
     * Function to log a message to a log store
     *
     * Intended usage:
     * Call this to log
     *
     * @param {string} logMessage - Required
     * @param {LoggerSeverities} severity - Required
     * @param {LoggerTeams} team
     *
     * @example
     * enum LoggerSeverities {
     *  ERROR = 'error',
     *  WARNING = 'warning',
     *  INFO = 'info',
     * };
     *
     * enum LoggerTeams {
     *  Team1 = 'Test Team 1',
     * };
     *
     * Logger.log({
     *  logMessage: 'There was an error',
     *  severity: LoggerSeverities.ERROR,
     *  team: LoggerTeams.Team1
     * });
     */
    log: ({ logMessage, severity, team }: Log<LoggerTeams, LoggerSeverities>) => void;
}
/**
 * Object representing Logger object and logging-related properties
 */
export interface LoggerOptions<LoggerTeams, LoggerSeverities> {
    /** A Logger */
    logger: Logger<LoggerTeams, LoggerSeverities>;
    /** The default team a log should be assigned to */
    defaultTeam: LoggerTeams;
}
/**
 * A Tracker object that can send data on client metrics.
 */
export interface Tracker<ExperimentViewedProps> {
    /**
     * @description
     * Function that broadcasts client-side analytics events
     *
     * Intended usage:
     * Used by the function `trackIsInSample` to broadcast 'Experiment Viewed' events
     *
     * @param {
     *   {
     *     eventName: 'Experiment Viewed',
     *     properties?: {},
     *     timeout?: number,
     *   }
     * } trackOptions - Required
     *
     * @return {Promise<void | Error | { message?: string }>}
     * Calling `track` will always return a promise.
     * - If the call is successful, the value of the promise will be void.
     * - If there is an error, the value of the promise will be the error.
     * - In some specific situations, the value of the promise will be an object with a key `message`.
     * -- {@link https://github.com/10gen/heliotrope|Heliotrope} returns these "objects with a message"
     * -- when it caches an analytics call (tries to make a track call before Heliotrope is
     * -- initialized), or when it attempts to make a track call when Heliotrope is not enabled.
     *
     * @example
     * `const trackPromise = this.#tracker.track({
     *   eventName: 'Experiment Viewed',
     *   properties: {
     *     customProperty: 'A Relevant Value',
     *   },
     *   timeout: 100,
     * });`
     *
     * Note: `this.#tracker.track()` will only be called via `ExperimentSDK.trackIsInSample({...})`, and not
     * by the SDK's consumers.
     *
     * Note: Any properties that are passed into the `properties` object will have to match the
     * `experimentViewedProps` type that was passed in when the SDK was initialized.
     *
     */
    track: (trackOptions: {
        eventName: 'Experiment Viewed';
        properties?: ExperimentViewedProps;
        timeout?: number;
    }) => Promise<void | Error | {
        message?: string;
    }>;
}
/**
 * Object representing Tracker object and tracking-related properties
 */
export interface TrackerOptions<ExperimentViewedProps> {
    /** A Tracker */
    tracker: Tracker<ExperimentViewedProps>;
}
/**
 * Default props for the "Experiment Viewed" event sent by trackIsInSample
 * Very similar to {@link ServerAssignment} except the keys are snake-case
 * and has 2 additional props: test_group_name and test_assignment
 */
export interface ExperimentViewedBaseEvent<T extends TypeData> {
    /**
     * See {@link ServerAssignment.testName} for more details
     *
     * @example
     * "ATLAS_SEARCH_IN_PROJECT_SIDE_NAV_20220620"
     */
    test_name: T['experimentNames'];
    /**
     * See {@link ServerAssignment.testGroupId} for more details. However,
     * 1 key difference is that this value will always be defined.
     *
     * Worth reiterating that this is NOT an ObjectId value and has the same value
     * as {@link ExperimentViewedBaseEvent.test_group_name}.
     *
     * @example
     * "atlasSearchProjectSideNavControl"
     */
    test_group_id: T['experimentVariantNames'];
    /**
     * Same as {@link ExperimentViewedBaseEvent.test_group_id}
     *
     * @example
     * "atlasSearchProjectSideNavControl"
     */
    test_group_name: T['experimentVariantNames'];
    /**
     * See {@link ServerAssignment.entityType} for more details.
     *
     * @example
     * "ORG"
     */
    entity_type: EntityType;
    /**
     * See {@link ServerAssignment.entityId} for more details.
     *
     * @example
     * "621509aae911cb2e37ba91ca"
     */
    entity_id: string;
    /**
     * See {@link ServerAssignment.tag} for more details.
     *
     * @example
     * "activation"
     */
    test_tag: string;
    /**
     * See {@link ServerAssignment.assignmentDate} for more details.
     *
     * @example
     * "1658340252128"
     */
    assignment_date: string;
    /**
     * See {@link ServerAssignment.testGroupDatabaseId} for more details. However,
     * 1 key difference is that this value will always be defined.
     *
     * @example
     * "62bb810b61be7d017fcc2196"
     */
    test_group_database_id: string;
    /**
     * See {@link ServerAssignment.testId} for more details.
     *
     * @example
     * "62bb810b61be7d017fcc2198"
     */
    test_id: string;
    /**
     * A formatted string that contains the experiment name and variant.
     *
     * @example
     * "test_ATLAS_SEARCH_IN_PROJECT_SIDE_NAV_20220620_variant_atlasSearchProjectSideNavControl"
     */
    test_assignment: string;
}
/**
 * Options to pass into the constructor
 */
export interface ConstructorOptions<T extends TypeData> {
    /** The entityId(s). Required. */
    entityIds: EntityIds;
    /** The environment the SDK is running in. Required. */
    environment: Environment;
    /** Logger-related properties */
    loggerOptions?: LoggerOptions<T['loggerTeams'], T['loggerSeverities']>;
    /** Tracker-related properties */
    trackerOptions?: TrackerOptions<T['experimentViewedProps']>;
}
/**
 * Response object from ExperimentSDK.getProperties
 * (Very similar to {@link ConstructorOptions} except Logger and TrackerOptions can be null)
 */
export interface GetPropertiesResponse<T extends TypeData> {
    /** The entityId(s) of the SDK instance. */
    entityIds: EntityIds;
    /** Logger-related properties of the SDK instance */
    loggerOptions: LoggerOptions<T['loggerTeams'], T['loggerSeverities']> | null;
    /** Tracker-related properties of the SDK instance */
    trackerOptions: TrackerOptions<T['experimentViewedProps']> | null;
    /** The environment of the SDK instance.. */
    environment: Environment;
}
/**
 * Object parameter for ExperimentSDK.addExistingExperimentData
 */
export interface ExistingData<T extends TypeData> {
    assignments?: Array<ServerAssignment<T['experimentNames'], T['experimentVariantNames']>>;
    experimentAttributes?: ServerExperimentAttributes<T['experimentAttributeNames']>;
}
/**
 * Object parameter for ExperimentSDK.addExperimentCalls, which is a dev-env-only function
 */
export interface AddExperimentCallsProps<T extends TypeData> {
    assignmentCalls?: Partial<Record<T['experimentNames'], Promise<AsyncStatus>>>;
    allocationPointCalls?: Partial<Record<T['allocationPointNames'], Promise<AsyncStatus>>>;
}
/**
 * Shared props that all functions can provide in their "options" objects
 */
interface BasicFunctionOptions<LoggerTeams> {
    /**
     * The "team" value that should be used for the logger
     * (if not passed, the "defaultTeam" from initialization is used)
     */
    team?: LoggerTeams;
}
/**
 * Represents additional options for ExperimentSDK.addExistingExperimentData
 */
export declare type AddExistingExperimentDataOptions<LoggerTeams> = BasicFunctionOptions<LoggerTeams>;
/**
 * Represents additional options for ExperimentSDK.getAssignments
 */
export interface GetAssignmentOptions<T extends TypeData> extends BasicFunctionOptions<T['loggerTeams']> {
    /**
     * Custom properties that will be passed to "trackIsInSample" to be used in the
     * "Experiment Viewed" call
     */
    trackIsInSampleCustomProps?: T['experimentViewedProps'];
}
/**
 * Represents additional options for ExperimentSDK.assign
 */
export interface AssignOptions<LoggerTeams> extends BasicFunctionOptions<LoggerTeams> {
    /**
     * An API timeout value, in milliseconds
     * (if not passed, a default value of 5 seconds is used)
     */
    timeoutMs?: number;
}
/**
 * Represents additional options for ExperimentSDK.assignByPoints
 * Right now it's aliased to the "AssignOptions" interface because the same options apply for this function.
 * However, in the future if the 2 option interfaces diverge, feel free to define this as its own
 * distinct interface.
 */
export declare type AssignByPointsOptions<LoggerTeams> = AssignOptions<LoggerTeams>;
/** API Types below */
/**
 * A typing to confirm that ROOT_URLS has values for all the known Environments
 */
export declare type RootURLMapping<Environment extends string> = {
    [key in Environment]: string;
};
/**
 * Object parameter for api.callMMS (an internal API helper function)
 */
export interface CallMMSProps {
    /**
     * Fully formed URI string
     */
    uri: string;
    /**
     * List of allowed REST actions. Add additional types as needed.
     */
    action: 'GET' | 'POST';
    /**
     * Optional - POST requests can have a body, but GET requests cannot.
     */
    body?: Record<string, Array<string>>;
    /**
     * An API timeout value, in milliseconds
     * (if not passed, a default value of 5 seconds is used)
     */
    timeoutMs?: number;
}
/**
 * Shared props for MMS API calls
 */
interface MMSApiProps {
    environment: Environment;
    groupId?: string;
    orgId?: string;
    timeoutMs?: number;
}
/**
 * Object parameter for api.mmsAssign
 */
export interface MMSAssignProps<T extends TypeData> extends MMSApiProps {
    experimentName: T['experimentNames'];
}
/**
 * Object parameter for api.mmsAssignByPoints
 */
export interface MMSAssignByPointsProps<T extends TypeData> extends MMSApiProps {
    allocationPoints: Array<T['allocationPointNames']>;
}
/**
 * Expected shape of the MMS BE assign/assignByPoint response
 */
export interface MMSAssignmentResponse<T extends TypeData> {
    /**
     * Represents experiment assignment info from MMS for an entity. Each entry corresponds to 1 experiment.
     */
    data: Array<ServerAssignment<T['experimentNames'], T['experimentVariantNames']>>;
    /**
     * Represents experimentAttribute values from MMS for an entity. An experiment can have multiple (or 0)
     * experimentAttribute values. There is no way to tell in the response payload
     * what experiment an experimentAttribute is tied to.
     */
    experimentFeatureFlags: ServerExperimentAttributes<T['experimentAttributeNames']>;
}
export {};
//# sourceMappingURL=types.d.ts.map